# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class OauthmodelClientRegisterResponse(Model):
    """Oauthmodel client register response (oauthmodel.ClientRegisterResponse)

    Properties:
        client_id: (client_id) REQUIRED str

        client_id_issued_at: (client_id_issued_at) REQUIRED int

        client_name: (client_name) REQUIRED str

        namespace: (namespace) REQUIRED str

        redirect_uris: (redirect_uris) REQUIRED List[str]

        client_uri: (client_uri) OPTIONAL str

        grant_types: (grant_types) OPTIONAL List[str]

        response_types: (response_types) OPTIONAL List[str]

        scopes: (scopes) OPTIONAL List[str]

        token_endpoint_auth_method: (token_endpoint_auth_method) OPTIONAL str
    """

    # region fields

    client_id: str  # REQUIRED
    client_id_issued_at: int  # REQUIRED
    client_name: str  # REQUIRED
    namespace: str  # REQUIRED
    redirect_uris: List[str]  # REQUIRED
    client_uri: str  # OPTIONAL
    grant_types: List[str]  # OPTIONAL
    response_types: List[str]  # OPTIONAL
    scopes: List[str]  # OPTIONAL
    token_endpoint_auth_method: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_client_id(self, value: str) -> OauthmodelClientRegisterResponse:
        self.client_id = value
        return self

    def with_client_id_issued_at(self, value: int) -> OauthmodelClientRegisterResponse:
        self.client_id_issued_at = value
        return self

    def with_client_name(self, value: str) -> OauthmodelClientRegisterResponse:
        self.client_name = value
        return self

    def with_namespace(self, value: str) -> OauthmodelClientRegisterResponse:
        self.namespace = value
        return self

    def with_redirect_uris(self, value: List[str]) -> OauthmodelClientRegisterResponse:
        self.redirect_uris = value
        return self

    def with_client_uri(self, value: str) -> OauthmodelClientRegisterResponse:
        self.client_uri = value
        return self

    def with_grant_types(self, value: List[str]) -> OauthmodelClientRegisterResponse:
        self.grant_types = value
        return self

    def with_response_types(self, value: List[str]) -> OauthmodelClientRegisterResponse:
        self.response_types = value
        return self

    def with_scopes(self, value: List[str]) -> OauthmodelClientRegisterResponse:
        self.scopes = value
        return self

    def with_token_endpoint_auth_method(
        self, value: str
    ) -> OauthmodelClientRegisterResponse:
        self.token_endpoint_auth_method = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "client_id"):
            result["client_id"] = str(self.client_id)
        elif include_empty:
            result["client_id"] = ""
        if hasattr(self, "client_id_issued_at"):
            result["client_id_issued_at"] = int(self.client_id_issued_at)
        elif include_empty:
            result["client_id_issued_at"] = 0
        if hasattr(self, "client_name"):
            result["client_name"] = str(self.client_name)
        elif include_empty:
            result["client_name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "redirect_uris"):
            result["redirect_uris"] = [str(i0) for i0 in self.redirect_uris]
        elif include_empty:
            result["redirect_uris"] = []
        if hasattr(self, "client_uri"):
            result["client_uri"] = str(self.client_uri)
        elif include_empty:
            result["client_uri"] = ""
        if hasattr(self, "grant_types"):
            result["grant_types"] = [str(i0) for i0 in self.grant_types]
        elif include_empty:
            result["grant_types"] = []
        if hasattr(self, "response_types"):
            result["response_types"] = [str(i0) for i0 in self.response_types]
        elif include_empty:
            result["response_types"] = []
        if hasattr(self, "scopes"):
            result["scopes"] = [str(i0) for i0 in self.scopes]
        elif include_empty:
            result["scopes"] = []
        if hasattr(self, "token_endpoint_auth_method"):
            result["token_endpoint_auth_method"] = str(self.token_endpoint_auth_method)
        elif include_empty:
            result["token_endpoint_auth_method"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        client_id: str,
        client_id_issued_at: int,
        client_name: str,
        namespace: str,
        redirect_uris: List[str],
        client_uri: Optional[str] = None,
        grant_types: Optional[List[str]] = None,
        response_types: Optional[List[str]] = None,
        scopes: Optional[List[str]] = None,
        token_endpoint_auth_method: Optional[str] = None,
        **kwargs,
    ) -> OauthmodelClientRegisterResponse:
        instance = cls()
        instance.client_id = client_id
        instance.client_id_issued_at = client_id_issued_at
        instance.client_name = client_name
        instance.namespace = namespace
        instance.redirect_uris = redirect_uris
        if client_uri is not None:
            instance.client_uri = client_uri
        if grant_types is not None:
            instance.grant_types = grant_types
        if response_types is not None:
            instance.response_types = response_types
        if scopes is not None:
            instance.scopes = scopes
        if token_endpoint_auth_method is not None:
            instance.token_endpoint_auth_method = token_endpoint_auth_method
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> OauthmodelClientRegisterResponse:
        instance = cls()
        if not dict_:
            return instance
        if "client_id" in dict_ and dict_["client_id"] is not None:
            instance.client_id = str(dict_["client_id"])
        elif include_empty:
            instance.client_id = ""
        if "client_id_issued_at" in dict_ and dict_["client_id_issued_at"] is not None:
            instance.client_id_issued_at = int(dict_["client_id_issued_at"])
        elif include_empty:
            instance.client_id_issued_at = 0
        if "client_name" in dict_ and dict_["client_name"] is not None:
            instance.client_name = str(dict_["client_name"])
        elif include_empty:
            instance.client_name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "redirect_uris" in dict_ and dict_["redirect_uris"] is not None:
            instance.redirect_uris = [str(i0) for i0 in dict_["redirect_uris"]]
        elif include_empty:
            instance.redirect_uris = []
        if "client_uri" in dict_ and dict_["client_uri"] is not None:
            instance.client_uri = str(dict_["client_uri"])
        elif include_empty:
            instance.client_uri = ""
        if "grant_types" in dict_ and dict_["grant_types"] is not None:
            instance.grant_types = [str(i0) for i0 in dict_["grant_types"]]
        elif include_empty:
            instance.grant_types = []
        if "response_types" in dict_ and dict_["response_types"] is not None:
            instance.response_types = [str(i0) for i0 in dict_["response_types"]]
        elif include_empty:
            instance.response_types = []
        if "scopes" in dict_ and dict_["scopes"] is not None:
            instance.scopes = [str(i0) for i0 in dict_["scopes"]]
        elif include_empty:
            instance.scopes = []
        if (
            "token_endpoint_auth_method" in dict_
            and dict_["token_endpoint_auth_method"] is not None
        ):
            instance.token_endpoint_auth_method = str(
                dict_["token_endpoint_auth_method"]
            )
        elif include_empty:
            instance.token_endpoint_auth_method = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, OauthmodelClientRegisterResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[OauthmodelClientRegisterResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        OauthmodelClientRegisterResponse,
        List[OauthmodelClientRegisterResponse],
        Dict[Any, OauthmodelClientRegisterResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "client_id": "client_id",
            "client_id_issued_at": "client_id_issued_at",
            "client_name": "client_name",
            "namespace": "namespace",
            "redirect_uris": "redirect_uris",
            "client_uri": "client_uri",
            "grant_types": "grant_types",
            "response_types": "response_types",
            "scopes": "scopes",
            "token_endpoint_auth_method": "token_endpoint_auth_method",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "client_id": True,
            "client_id_issued_at": True,
            "client_name": True,
            "namespace": True,
            "redirect_uris": True,
            "client_uri": False,
            "grant_types": False,
            "response_types": False,
            "scopes": False,
            "token_endpoint_auth_method": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the Custom Service Manager."""

__version__ = "1.34.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .apimodel_app_debug_info_response import ApimodelAppDebugInfoResponse
from .apimodel_app_item import ApimodelAppItem
from .apimodel_app_item_v5 import ApimodelAppItemV5
from .apimodel_app_status_progress_step import ApimodelAppStatusProgressStep
from .apimodel_app_ui_response import ApimodelAppUIResponse
from .apimodel_apply_app_config_request import ApimodelApplyAppConfigRequest
from .apimodel_apply_config_item import ApimodelApplyConfigItem
from .apimodel_apply_permission_item import ApimodelApplyPermissionItem
from .apimodel_autoscaling_request import ApimodelAutoscalingRequest
from .apimodel_autoscaling_response import ApimodelAutoscalingResponse
from .apimodel_bulk_subscribe_item import ApimodelBulkSubscribeItem
from .apimodel_bulk_subscribe_request import ApimodelBulkSubscribeRequest
from .apimodel_cpu_request import ApimodelCPURequest
from .apimodel_cpu_response import ApimodelCPUResponse
from .apimodel_create_app_ui_request import ApimodelCreateAppUIRequest
from .apimodel_create_app_v2_request import ApimodelCreateAppV2Request
from .apimodel_create_app_v5_request import ApimodelCreateAppV5Request
from .apimodel_create_deployment_v2_request import ApimodelCreateDeploymentV2Request
from .apimodel_create_deployment_v2_response import ApimodelCreateDeploymentV2Response
from .apimodel_create_key_value_credential_request import (
    ApimodelCreateKeyValueCredentialRequest,
)
from .apimodel_create_key_value_credential_response import (
    ApimodelCreateKeyValueCredentialResponse,
)
from .apimodel_create_no_sql_app_database_request import (
    ApimodelCreateNoSQLAppDatabaseRequest,
)
from .apimodel_create_no_sql_database_credential_request import (
    ApimodelCreateNoSQLDatabaseCredentialRequest,
)
from .apimodel_create_sql_app_database_request import (
    ApimodelCreateSQLAppDatabaseRequest,
)
from .apimodel_create_sql_database_credential_request import (
    ApimodelCreateSQLDatabaseCredentialRequest,
)
from .apimodel_create_subscription_request import ApimodelCreateSubscriptionRequest
from .apimodel_create_subscription_response import ApimodelCreateSubscriptionResponse
from .apimodel_create_topic_request import ApimodelCreateTopicRequest
from .apimodel_csm_app_limits_response import ApimodelCSMAppLimitsResponse
from .apimodel_delete_no_sql_resource_response import (
    ApimodelDeleteNoSQLResourceResponse,
)
from .apimodel_delete_sql_resource_response import ApimodelDeleteSQLResourceResponse
from .apimodel_get_app_image_list_v2_data_item import ApimodelGetAppImageListV2DataItem
from .apimodel_get_app_image_list_v2_data_item_image_scan_result import (
    ApimodelGetAppImageListV2DataItemImageScanResult,
)
from .apimodel_get_app_image_list_v2_data_item_image_scan_status import (
    ApimodelGetAppImageListV2DataItemImageScanStatus,
)
from .apimodel_get_app_image_list_v2_response import ApimodelGetAppImageListV2Response
from .apimodel_get_app_list_v2_request import ApimodelGetAppListV2Request
from .apimodel_get_app_list_v2_response import ApimodelGetAppListV2Response
from .apimodel_get_app_status_progress_response import (
    ApimodelGetAppStatusProgressResponse,
)
from .apimodel_get_deployment_list_v2_data_item import (
    ApimodelGetDeploymentListV2DataItem,
)
from .apimodel_get_deployment_list_v2_request import ApimodelGetDeploymentListV2Request
from .apimodel_get_deployment_list_v2_response import (
    ApimodelGetDeploymentListV2Response,
)
from .apimodel_get_integration_app_key_value_response import (
    ApimodelGetIntegrationAppKeyValueResponse,
)
from .apimodel_get_list_of_configurations_v2_data_item import (
    ApimodelGetListOfConfigurationsV2DataItem,
)
from .apimodel_get_list_of_configurations_v2_response import (
    ApimodelGetListOfConfigurationsV2Response,
)
from .apimodel_get_notification_subscriber_item import (
    ApimodelGetNotificationSubscriberItem,
)
from .apimodel_get_notification_subscriber_item_v2 import (
    ApimodelGetNotificationSubscriberItemV2,
)
from .apimodel_get_notification_subscriber_list_response import (
    ApimodelGetNotificationSubscriberListResponse,
)
from .apimodel_get_notification_subscriber_list_response_v2 import (
    ApimodelGetNotificationSubscriberListResponseV2,
)
from .apimodel_get_notification_subscriber_status_response import (
    ApimodelGetNotificationSubscriberStatusResponse,
)
from .apimodel_increase_limit_form_request import ApimodelIncreaseLimitFormRequest
from .apimodel_integrate_app_key_value_request import (
    ApimodelIntegrateAppKeyValueRequest,
)
from .apimodel_integrate_app_key_value_response import (
    ApimodelIntegrateAppKeyValueResponse,
)
from .apimodel_integration_credential_info import ApimodelIntegrationCredentialInfo
from .apimodel_integration_list_item import ApimodelIntegrationListItem
from .apimodel_key_value_credential_acknowledgements import (
    ApimodelKeyValueCredentialAcknowledgements,
)
from .apimodel_key_value_resource_list_response import (
    ApimodelKeyValueResourceListResponse,
)
from .apimodel_key_value_resource_response import ApimodelKeyValueResourceResponse
from .apimodel_list_app_ui_response import ApimodelListAppUIResponse
from .apimodel_list_integrated_apps_key_value_response import (
    ApimodelListIntegratedAppsKeyValueResponse,
)
from .apimodel_list_topics_response import ApimodelListTopicsResponse
from .apimodel_memory_request import ApimodelMemoryRequest
from .apimodel_memory_response import ApimodelMemoryResponse
from .apimodel_no_sql_app_list_response import ApimodelNoSQLAppListResponse
from .apimodel_no_sql_app_response import ApimodelNoSQLAppResponse
from .apimodel_no_sql_database_acknowledgements import (
    ApimodelNoSQLDatabaseAcknowledgements,
)
from .apimodel_no_sql_database_credential_response import (
    ApimodelNoSQLDatabaseCredentialResponse,
)
from .apimodel_no_sql_database_delete_response import (
    ApimodelNoSQLDatabaseDeleteResponse,
)
from .apimodel_no_sql_database_response import ApimodelNoSQLDatabaseResponse
from .apimodel_no_sql_resource_response import ApimodelNoSQLResourceResponse
from .apimodel_notification_type import ApimodelNotificationType
from .apimodel_pagination_response import ApimodelPaginationResponse
from .apimodel_public_iam_client_response import ApimodelPublicIAMClientResponse
from .apimodel_replica_request import ApimodelReplicaRequest
from .apimodel_replica_response import ApimodelReplicaResponse
from .apimodel_save_configuration_v2_request import ApimodelSaveConfigurationV2Request
from .apimodel_save_configuration_v2_response import ApimodelSaveConfigurationV2Response
from .apimodel_save_secret_configuration_v2_request import (
    ApimodelSaveSecretConfigurationV2Request,
)
from .apimodel_self_subscribe_notification_request import (
    ApimodelSelfSubscribeNotificationRequest,
)
from .apimodel_sql_app_list_response import ApimodelSQLAppListResponse
from .apimodel_sql_app_response import ApimodelSQLAppResponse
from .apimodel_sql_database_acknowledgements import ApimodelSQLDatabaseAcknowledgements
from .apimodel_sql_database_credential_response import (
    ApimodelSQLDatabaseCredentialResponse,
)
from .apimodel_sql_database_delete_response import ApimodelSQLDatabaseDeleteResponse
from .apimodel_sql_database_response import ApimodelSQLDatabaseResponse
from .apimodel_sql_resource_response import ApimodelSQLResourceResponse
from .apimodel_subscribe_notification_request import (
    ApimodelSubscribeNotificationRequest,
)
from .apimodel_subscribe_notification_response import (
    ApimodelSubscribeNotificationResponse,
)
from .apimodel_subscriber_item_request import ApimodelSubscriberItemRequest
from .apimodel_subscriber_item_response import ApimodelSubscriberItemResponse
from .apimodel_topic_detail_response import ApimodelTopicDetailResponse
from .apimodel_topic_response import ApimodelTopicResponse
from .apimodel_topic_subscription import ApimodelTopicSubscription
from .apimodel_tunnel_info_response import ApimodelTunnelInfoResponse
from .apimodel_update_app_resource_request import ApimodelUpdateAppResourceRequest
from .apimodel_update_app_v2_request import ApimodelUpdateAppV2Request
from .apimodel_update_configuration_v2_request import (
    ApimodelUpdateConfigurationV2Request,
)
from .apimodel_update_configuration_v2_response import (
    ApimodelUpdateConfigurationV2Response,
)
from .apimodel_update_debug_mode_request import ApimodelUpdateDebugModeRequest
from .apimodel_update_secret_configuration_v2_request import (
    ApimodelUpdateSecretConfigurationV2Request,
)
from .apimodel_upload_file_response import ApimodelUploadFileResponse
from .createappparams_autoscaling_request import CreateappparamsAutoscalingRequest
from .createappparams_cpu_request import CreateappparamsCPURequest
from .createappparams_memory_request import CreateappparamsMemoryRequest
from .createappparams_replica_request import CreateappparamsReplicaRequest
from .domain_allowed_intercepted_port import DomainAllowedInterceptedPort
from .domain_debug_pod import DomainDebugPod
from .domain_exposed_service import DomainExposedService
from .domain_key_value_cluster_config import DomainKeyValueClusterConfig
from .domain_key_value_resource_configuration import DomainKeyValueResourceConfiguration
from .domain_key_value_update_configuration import DomainKeyValueUpdateConfiguration
from .generated_create_app_v1_request import GeneratedCreateAppV1Request
from .generated_create_app_v1_response import GeneratedCreateAppV1Response
from .generated_create_deployment_v1_request import GeneratedCreateDeploymentV1Request
from .generated_create_deployment_v1_response import GeneratedCreateDeploymentV1Response
from .generated_delete_app_images_v1_request import GeneratedDeleteAppImagesV1Request
from .generated_get_app_image_list_v1_data_item import (
    GeneratedGetAppImageListV1DataItem,
)
from .generated_get_app_image_list_v1_data_item_image_scan_status import (
    GeneratedGetAppImageListV1DataItemImageScanStatus,
)
from .generated_get_app_image_list_v1_response import GeneratedGetAppImageListV1Response
from .generated_get_app_list_v1_data_item import GeneratedGetAppListV1DataItem
from .generated_get_app_list_v1_request import GeneratedGetAppListV1Request
from .generated_get_app_list_v1_response import GeneratedGetAppListV1Response
from .generated_get_app_release_v1_response import GeneratedGetAppReleaseV1Response
from .generated_get_app_v1_response import GeneratedGetAppV1Response
from .generated_get_deployment_list_v1_data_item import (
    GeneratedGetDeploymentListV1DataItem,
)
from .generated_get_deployment_list_v1_request import (
    GeneratedGetDeploymentListV1Request,
)
from .generated_get_deployment_list_v1_response import (
    GeneratedGetDeploymentListV1Response,
)
from .generated_get_deployment_v1_response import GeneratedGetDeploymentV1Response
from .generated_get_list_of_configurations_v1_data_item import (
    GeneratedGetListOfConfigurationsV1DataItem,
)
from .generated_get_list_of_configurations_v1_response import (
    GeneratedGetListOfConfigurationsV1Response,
)
from .generated_pagination import GeneratedPagination
from .generated_save_configuration_v1_request import GeneratedSaveConfigurationV1Request
from .generated_save_configuration_v1_response import (
    GeneratedSaveConfigurationV1Response,
)
from .generated_start_app_v1_request import GeneratedStartAppV1Request
from .generated_start_app_v1_response import GeneratedStartAppV1Response
from .generated_stop_app_v1_request import GeneratedStopAppV1Request
from .generated_stop_app_v1_response import GeneratedStopAppV1Response
from .generated_update_app_v1_request import GeneratedUpdateAppV1Request
from .generated_update_app_v1_response import GeneratedUpdateAppV1Response
from .generated_update_configuration_v1_request import (
    GeneratedUpdateConfigurationV1Request,
)
from .generated_update_configuration_v1_response import (
    GeneratedUpdateConfigurationV1Response,
)
from .log_app_message_declaration import LogAppMessageDeclaration
from .model_csm_autoscaling_defaults import ModelCSMAutoscalingDefaults
from .model_image_scan_finding import ModelImageScanFinding
from .nosqlresource_no_sql_resource_configuration import (
    NosqlresourceNoSQLResourceConfiguration,
)
from .resourceaccesstunnel_tunnel_info import ResourceaccesstunnelTunnelInfo
from .response_error_response import ResponseErrorResponse
from .sqlresource_sql_resource_configuration import SqlresourceSQLResourceConfiguration



# Copyright (c) 2025 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from abc import ABC, abstractmethod
from dataclasses import dataclass
from threading import RLock
from typing import Callable, Optional, Tuple

from ...core import (
    AccelByteSDK,
    HttpResponse,
)

from ._login_const import (
    LOGIN_TYPE_CLIENT,
    LOGIN_TYPE_PLATFORM,
    LOGIN_TYPE_USER,
    LOGIN_TYPE_REFRESHABLE,
    DEFAULT_MAX_RETRY,
    DEFAULT_REFRESH_RATE,
    REFRESH_KWARGS,
)

from ._login_refresh import (
    try_refresh_login_internal,
    try_refresh_login_async_internal,
)

from .v2 import (
    login_client,
    login_client_async,
)


TokenRefresherResult = Tuple[bool, Optional[HttpResponse]]
TokenRefresherFailureFunc = Optional[Callable[[HttpResponse], None]]


@dataclass
class TokenRefresherOptions:
    enabled: bool = False
    max_retry: int = DEFAULT_MAX_RETRY
    refresh_rate: float = DEFAULT_REFRESH_RATE


class TokenRefresherBase(ABC):
    def __init__(
        self, *args, options: Optional[TokenRefresherOptions] = None, **kwargs
    ) -> None:
        self._lock: RLock = RLock()
        self._options: TokenRefresherOptions = options or TokenRefresherOptions()
        self._on_failure: TokenRefresherFailureFunc = None

    @abstractmethod
    def refresh(self, sdk: AccelByteSDK) -> TokenRefresherResult:
        pass

    @abstractmethod
    async def refresh_async(self, sdk: AccelByteSDK) -> TokenRefresherResult:
        pass


class TokenRefresher(TokenRefresherBase):
    def refresh(self, sdk: AccelByteSDK) -> TokenRefresherResult:
        if not self._options.enabled:
            return False, None

        token_repo = sdk.get_token_repository(raise_when_none=False)
        if not token_repo:
            return HttpResponse.create_token_repo_not_found_error()

        if not token_repo.has_token():
            return HttpResponse.create_token_not_found_error()

        with self._lock:
            token_type = token_repo.get_token_type()
            refresh_token = token_repo.get_refresh_token()
            refresh_rate: float = self._options.refresh_rate
            is_token_expired: bool = token_repo.has_token_expired(
                multiplier=refresh_rate
            )
            if not is_token_expired:
                return False, None

            number_of_attempts: int = 0
            max_retry: int = self._options.max_retry
            error: Optional[HttpResponse] = None

            while True:
                # noinspection PyBroadException
                try:
                    if token_type == LOGIN_TYPE_CLIENT:
                        _, error = login_client(sdk=sdk, **REFRESH_KWARGS)
                        if not error:
                            return True, None
                    elif token_type in (
                        LOGIN_TYPE_PLATFORM,
                        LOGIN_TYPE_USER,
                        LOGIN_TYPE_REFRESHABLE,
                    ):
                        _, error = try_refresh_login_internal(
                            refresh_rate=refresh_rate, sdk=sdk, **REFRESH_KWARGS
                        )
                        if not error:
                            return True, None
                except Exception as exception:
                    error = HttpResponse.create_error(code=500, error=str(exception))

                number_of_attempts += 1
                if number_of_attempts >= max_retry:
                    break

        if self._on_failure:
            # pylint: disable=not-callable
            self._on_failure(error)

        return False, error

    async def refresh_async(self, sdk: AccelByteSDK) -> TokenRefresherResult:
        if not self._options.enabled:
            return False, None

        token_repo = sdk.get_token_repository(raise_when_none=False)
        if not token_repo:
            return HttpResponse.create_token_repo_not_found_error()

        if not token_repo.has_token():
            return HttpResponse.create_token_not_found_error()

        with self._lock:
            token_type = token_repo.get_token_type()
            refresh_token = token_repo.get_refresh_token()
            refresh_rate: float = self._options.refresh_rate
            is_token_expired: bool = token_repo.has_token_expired(
                multiplier=refresh_rate
            )
            if not is_token_expired:
                return False, None

            number_of_attempts: int = 0
            max_retry: int = self._options.max_retry
            error: Optional[HttpResponse] = None

            while True:
                # noinspection PyBroadException
                try:
                    if token_type == LOGIN_TYPE_CLIENT:
                        _, error = await login_client_async(sdk=sdk, **REFRESH_KWARGS)
                        if not error:
                            return True, None
                    elif token_type in (
                        LOGIN_TYPE_PLATFORM,
                        LOGIN_TYPE_USER,
                        LOGIN_TYPE_REFRESHABLE,
                    ):
                        _, error = await try_refresh_login_async_internal(
                            refresh_rate=refresh_rate,
                            sdk=sdk,
                            **REFRESH_KWARGS,
                        )
                        if not error:
                            return True, None
                except Exception as exception:
                    error = HttpResponse.create_error(code=500, error=str(exception))

                number_of_attempts += 1
                if number_of_attempts >= max_retry:
                    break

        if self._on_failure:
            # pylint: disable=not-callable
            self._on_failure(error)

        return False, error


__all__ = [
    "TokenRefresherBase",
    "TokenRefresher",
    "TokenRefresherOptions",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Reporting Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.reporting.models import RestapiActionApiRequest
from ..api.reporting.models import RestapiActionApiResponse
from ..api.reporting.models import RestapiActionListApiResponse
from ..api.reporting.models import RestapiAdminAllReasonsResponse
from ..api.reporting.models import RestapiAdminReasonListResponse
from ..api.reporting.models import RestapiAdminReasonResponse
from ..api.reporting.models import RestapiBanAccountActionRequest
from ..api.reporting.models import RestapiBanAccountActionResponse
from ..api.reporting.models import RestapiCategoryLimit
from ..api.reporting.models import RestapiConfigResponse
from ..api.reporting.models import RestapiCreateReasonGroupRequest
from ..api.reporting.models import RestapiCreateReasonRequest
from ..api.reporting.models import RestapiErrorResponse
from ..api.reporting.models import RestapiExtensionCategoryApiRequest
from ..api.reporting.models import RestapiExtensionCategoryApiResponse
from ..api.reporting.models import RestapiExtensionCategoryListApiResponse
from ..api.reporting.models import RestapiModerationRuleActionsRequest
from ..api.reporting.models import RestapiModerationRuleActionsResponse
from ..api.reporting.models import RestapiModerationRuleActiveRequest
from ..api.reporting.models import RestapiModerationRuleRequest
from ..api.reporting.models import RestapiModerationRuleResponse
from ..api.reporting.models import RestapiModerationRulesList
from ..api.reporting.models import RestapiPagination
from ..api.reporting.models import RestapiPublicReasonGroupResponse
from ..api.reporting.models import RestapiPublicReasonListResponse
from ..api.reporting.models import RestapiPublicReasonResponse
from ..api.reporting.models import RestapiReasonGroupListResponse
from ..api.reporting.models import RestapiReasonGroupResponse
from ..api.reporting.models import RestapiReportListResponse
from ..api.reporting.models import RestapiReportResponse
from ..api.reporting.models import RestapiReportingLimit
from ..api.reporting.models import RestapiSubmitReportRequest
from ..api.reporting.models import RestapiSubmitReportResponse
from ..api.reporting.models import RestapiTicketListResponse
from ..api.reporting.models import RestapiTicketResponse
from ..api.reporting.models import RestapiTicketStatisticResponse
from ..api.reporting.models import RestapiUnusedReasonListResponse
from ..api.reporting.models import RestapiUpdateReasonGroupRequest
from ..api.reporting.models import RestapiUpdateTicketResolutionsRequest


def create_restapi_action_api_request_example() -> RestapiActionApiRequest:
    instance = RestapiActionApiRequest()
    instance.action_id = randomize()
    instance.action_name = randomize()
    instance.event_name = randomize()
    return instance


def create_restapi_action_api_response_example() -> RestapiActionApiResponse:
    instance = RestapiActionApiResponse()
    instance.action_id = randomize()
    instance.action_name = randomize()
    instance.event_name = randomize()
    return instance


def create_restapi_action_list_api_response_example() -> RestapiActionListApiResponse:
    instance = RestapiActionListApiResponse()
    instance.data = [create_restapi_action_api_response_example()]
    return instance


def create_restapi_admin_all_reasons_response_example() -> (
    RestapiAdminAllReasonsResponse
):
    instance = RestapiAdminAllReasonsResponse()
    instance.data = [create_restapi_public_reason_response_example()]
    return instance


def create_restapi_admin_reason_list_response_example() -> (
    RestapiAdminReasonListResponse
):
    instance = RestapiAdminReasonListResponse()
    instance.data = [create_restapi_admin_reason_response_example()]
    instance.paging = create_restapi_pagination_example()
    return instance


def create_restapi_admin_reason_response_example() -> RestapiAdminReasonResponse:
    instance = RestapiAdminReasonResponse()
    instance.created_at = randomize()
    instance.description = randomize()
    instance.id_ = randomize()
    instance.namespace = randomize("slug")
    instance.title = randomize()
    instance.updated_at = randomize()
    instance.groups = [create_restapi_public_reason_group_response_example()]
    return instance


def create_restapi_ban_account_action_request_example() -> (
    RestapiBanAccountActionRequest
):
    instance = RestapiBanAccountActionRequest()
    instance.duration = randomize("int", min_val=1, max_val=1000)
    instance.reason = randomize()
    instance.skip_notif = randomize("bool")
    instance.type_ = randomize()
    instance.comment = randomize()
    return instance


def create_restapi_ban_account_action_response_example() -> (
    RestapiBanAccountActionResponse
):
    instance = RestapiBanAccountActionResponse()
    instance.duration = randomize("int", min_val=1, max_val=1000)
    instance.reason = randomize()
    instance.skip_notif = randomize("bool")
    instance.type_ = randomize()
    instance.comment = randomize()
    return instance


def create_restapi_category_limit_example() -> RestapiCategoryLimit:
    instance = RestapiCategoryLimit()
    instance.max_report_per_ticket = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.extension_category = randomize()
    return instance


def create_restapi_config_response_example() -> RestapiConfigResponse:
    instance = RestapiConfigResponse()
    instance.namespace = randomize("slug")
    instance.reporting_limit = create_restapi_reporting_limit_example()
    instance.updated_at = randomize("date")
    return instance


def create_restapi_create_reason_group_request_example() -> (
    RestapiCreateReasonGroupRequest
):
    instance = RestapiCreateReasonGroupRequest()
    instance.title = randomize()
    instance.reason_ids = [randomize()]
    return instance


def create_restapi_create_reason_request_example() -> RestapiCreateReasonRequest:
    instance = RestapiCreateReasonRequest()
    instance.description = randomize()
    instance.group_ids = [randomize()]
    instance.title = randomize()
    return instance


def create_restapi_error_response_example() -> RestapiErrorResponse:
    instance = RestapiErrorResponse()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance


def create_restapi_extension_category_api_request_example() -> (
    RestapiExtensionCategoryApiRequest
):
    instance = RestapiExtensionCategoryApiRequest()
    instance.extension_category = randomize()
    instance.extension_category_name = randomize()
    instance.service_source = randomize()
    return instance


def create_restapi_extension_category_api_response_example() -> (
    RestapiExtensionCategoryApiResponse
):
    instance = RestapiExtensionCategoryApiResponse()
    instance.extension_category = randomize()
    instance.extension_category_name = randomize()
    instance.service_source = randomize()
    return instance


def create_restapi_extension_category_list_api_response_example() -> (
    RestapiExtensionCategoryListApiResponse
):
    instance = RestapiExtensionCategoryListApiResponse()
    instance.data = [create_restapi_extension_category_api_response_example()]
    return instance


def create_restapi_moderation_rule_actions_request_example() -> (
    RestapiModerationRuleActionsRequest
):
    instance = RestapiModerationRuleActionsRequest()
    instance.auto_resolve = randomize("bool")
    instance.ban_account = create_restapi_ban_account_action_request_example()
    instance.delete_chat = randomize("bool")
    instance.extension_action_ids = [randomize()]
    instance.hide_content = randomize("bool")
    return instance


def create_restapi_moderation_rule_actions_response_example() -> (
    RestapiModerationRuleActionsResponse
):
    instance = RestapiModerationRuleActionsResponse()
    instance.auto_resolve = randomize("bool")
    instance.ban_account = create_restapi_ban_account_action_response_example()
    instance.delete_chat = randomize("bool")
    instance.extension_action_ids = [randomize()]
    instance.hide_content = randomize("bool")
    return instance


def create_restapi_moderation_rule_active_request_example() -> (
    RestapiModerationRuleActiveRequest
):
    instance = RestapiModerationRuleActiveRequest()
    instance.active = randomize("bool")
    return instance


def create_restapi_moderation_rule_request_example() -> RestapiModerationRuleRequest:
    instance = RestapiModerationRuleRequest()
    instance.active = randomize("bool")
    instance.category = randomize()
    instance.reason = randomize()
    instance.threshold = randomize("int", min_val=1, max_val=1000)
    instance.action = randomize()
    instance.actions = create_restapi_moderation_rule_actions_request_example()
    instance.extension_category = randomize()
    return instance


def create_restapi_moderation_rule_response_example() -> RestapiModerationRuleResponse:
    instance = RestapiModerationRuleResponse()
    instance.actions = create_restapi_moderation_rule_actions_response_example()
    instance.active = randomize("bool")
    instance.category = randomize()
    instance.created_at = randomize("date")
    instance.id_ = randomize()
    instance.namespace = randomize("slug")
    instance.reason = randomize()
    instance.threshold = randomize("int", min_val=1, max_val=1000)
    instance.action = randomize()
    instance.extension_category = randomize()
    instance.updated_at = randomize("date")
    return instance


def create_restapi_moderation_rules_list_example() -> RestapiModerationRulesList:
    instance = RestapiModerationRulesList()
    instance.data = [create_restapi_moderation_rule_response_example()]
    instance.paging = create_restapi_pagination_example()
    return instance


def create_restapi_pagination_example() -> RestapiPagination:
    instance = RestapiPagination()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_restapi_public_reason_group_response_example() -> (
    RestapiPublicReasonGroupResponse
):
    instance = RestapiPublicReasonGroupResponse()
    instance.id_ = randomize()
    instance.title = randomize()
    return instance


def create_restapi_public_reason_list_response_example() -> (
    RestapiPublicReasonListResponse
):
    instance = RestapiPublicReasonListResponse()
    instance.data = [create_restapi_public_reason_response_example()]
    instance.paging = create_restapi_pagination_example()
    return instance


def create_restapi_public_reason_response_example() -> RestapiPublicReasonResponse:
    instance = RestapiPublicReasonResponse()
    instance.description = randomize()
    instance.title = randomize()
    return instance


def create_restapi_reason_group_list_response_example() -> (
    RestapiReasonGroupListResponse
):
    instance = RestapiReasonGroupListResponse()
    instance.data = [create_restapi_public_reason_group_response_example()]
    instance.paging = create_restapi_pagination_example()
    return instance


def create_restapi_reason_group_response_example() -> RestapiReasonGroupResponse:
    instance = RestapiReasonGroupResponse()
    instance.id_ = randomize()
    instance.title = randomize()
    instance.reasons = [create_restapi_admin_reason_response_example()]
    return instance


def create_restapi_report_list_response_example() -> RestapiReportListResponse:
    instance = RestapiReportListResponse()
    instance.data = [create_restapi_report_response_example()]
    instance.paging = create_restapi_pagination_example()
    return instance


def create_restapi_report_response_example() -> RestapiReportResponse:
    instance = RestapiReportResponse()
    instance.additional_info = {randomize(): randomize()}
    instance.category = randomize()
    instance.comment = randomize()
    instance.created_at = randomize()
    instance.id_ = randomize()
    instance.namespace = randomize("slug")
    instance.object_id = randomize()
    instance.object_type = randomize()
    instance.reason = randomize()
    instance.reporter_id = randomize()
    instance.ticket_id = randomize()
    instance.updated_at = randomize()
    instance.user_id = randomize("uid")
    instance.extension_category = randomize()
    return instance


def create_restapi_reporting_limit_example() -> RestapiReportingLimit:
    instance = RestapiReportingLimit()
    instance.category_limits = [create_restapi_category_limit_example()]
    instance.time_interval = randomize("int", min_val=1, max_val=1000)
    instance.user_max_report_per_time_interval = randomize(
        "int", min_val=1, max_val=1000
    )
    return instance


def create_restapi_submit_report_request_example() -> RestapiSubmitReportRequest:
    instance = RestapiSubmitReportRequest()
    instance.category = randomize()
    instance.reason = randomize()
    instance.user_id = randomize("uid")
    instance.additional_info = {randomize(): randomize()}
    instance.comment = randomize()
    instance.extension_category = randomize()
    instance.object_id = randomize()
    instance.object_type = randomize()
    return instance


def create_restapi_submit_report_response_example() -> RestapiSubmitReportResponse:
    instance = RestapiSubmitReportResponse()
    instance.category = randomize()
    instance.created_at = randomize()
    instance.namespace = randomize("slug")
    instance.object_id = randomize()
    instance.object_type = randomize()
    instance.status = randomize()
    instance.ticket_id = randomize()
    instance.updated_at = randomize()
    instance.user_id = randomize("uid")
    instance.extension_category = randomize()
    return instance


def create_restapi_ticket_list_response_example() -> RestapiTicketListResponse:
    instance = RestapiTicketListResponse()
    instance.data = [create_restapi_ticket_response_example()]
    instance.paging = create_restapi_pagination_example()
    return instance


def create_restapi_ticket_response_example() -> RestapiTicketResponse:
    instance = RestapiTicketResponse()
    instance.category = randomize()
    instance.created_at = randomize()
    instance.id_ = randomize()
    instance.namespace = randomize("slug")
    instance.notes = randomize()
    instance.object_id = randomize()
    instance.object_type = randomize()
    instance.reports_count = randomize("int", min_val=1, max_val=1000)
    instance.status = randomize()
    instance.updated_at = randomize()
    instance.user_id = randomize("uid")
    instance.extension_category = randomize()
    return instance


def create_restapi_ticket_statistic_response_example() -> (
    RestapiTicketStatisticResponse
):
    instance = RestapiTicketStatisticResponse()
    instance.moderated_count = randomize("int", min_val=1, max_val=1000)
    instance.open_count = randomize("int", min_val=1, max_val=1000)
    instance.total_count = randomize("int", min_val=1, max_val=1000)
    return instance


def create_restapi_unused_reason_list_response_example() -> (
    RestapiUnusedReasonListResponse
):
    instance = RestapiUnusedReasonListResponse()
    instance.reasons = [create_restapi_public_reason_response_example()]
    return instance


def create_restapi_update_reason_group_request_example() -> (
    RestapiUpdateReasonGroupRequest
):
    instance = RestapiUpdateReasonGroupRequest()
    instance.title = randomize()
    instance.reason_ids = [randomize()]
    return instance


def create_restapi_update_ticket_resolutions_request_example() -> (
    RestapiUpdateTicketResolutionsRequest
):
    instance = RestapiUpdateTicketResolutionsRequest()
    instance.notes = randomize()
    instance.status = randomize()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Platform Service."""

__version__ = "6.14.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .download_invoice_details import DownloadInvoiceDetails
from .download_invoice_details import ItemTypeEnum as DownloadInvoiceDetailsItemTypeEnum
from .generate_invoice_summary import GenerateInvoiceSummary
from .generate_invoice_summary import ItemTypeEnum as GenerateInvoiceSummaryItemTypeEnum



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Session Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ApimodelsCreatePartyRequest
from ...models import ApimodelsPartySessionResponse
from ...models import ResponseError


class PublicCreateParty(Operation):
    """Create a party. (publicCreateParty)

    A join code will be autogenerated if the party is joinable.

    A user can be in 1 party at a time, therefore the requester will be removed from their previous party (if any) and automatically added/joined into this newly created party as a leader.

    Session configuration name is optional.
    Default configuration name if empty:
    {
    "name": "default",
    "type": "NONE",
    "joinability": "OPEN",
    "minPlayers": 1,
    "maxPlayers": 8,
    "inviteTimeout": 60,
    "inactiveTimeout": 60,
    "textChat": false
    }
    When session configuration "name" is provided, we will refer to the template if these fields are empty:
    - textChat
    - minPlayers
    - maxPlayers
    - inviteTimeout
    - inactiveTimeout
    - type
    - joinability
    - configurationName
    - attributes

    maxPlayers override rules:
    - maxPlayers can be set to a value lower than or equal to the template's maxPlayers. Values above the template ceiling are ignored and the template value is used instead.

    Supported platforms:
    1. STEAM
    2. PSN
    3. XBOX

    Reserved attributes key:
    1. preference: used to store preference of the leader and it is non-replaceable to keep the initial behavior of
    the session regardless the leader changes.
    2. NATIVESESSIONTITLE: used for session sync, to define name of session displayed on PlayStation system UI.

    Session has 2 fields for user status: "status" and "statusV2". The "status" is there for backward-compatibility, therefore we encourage to just rely on "statusV2" for the more updated statuses.

    User statuses:
    1. INVITED: by default, to join a session (except session with OPEN joinability or if session configuration has "autoJoin" sets to True) a user will receive an invite. The invitee will have the chance to respond within "inviteTimeout" which you can configure through session configuration.
    2. TIMEOUT: when a user is invited to a session, they will receive an invite. Unless "disableResendInvite" sets to True in the session configuration, the user will also receive invite reminder every 30s until they respond to the invite.
    3. REJECTED: when a user rejects an invite. To rejoin an INVITE_ONLY session, they will need to be re-invited and accept the invite.
    4. JOINED: there are few ways of a user to join a session, by invite, direct join (depends on session joinability) or join by code. upon invite, once a user accepts an invite, their status will be changed to JOINED.
    5. LEFT: user can leave a session. in case of party, a user can only be in 1 party at a time. therefore when they decide to create or join another party, they will be automatically removed from their initial party and their status will be changed to LEFT.
    6. KICKED: only party leader can kick a member.
    7. DISCONNECTED: if user still have reserved seat in the session and they disconnect lobby websocket, their status in the session will be changed to DISCONNECTED and field "members.previousStatus" of that user will contains the initial status before they disconnect lobby websocket. the user will be given chance to reconnect within "inactiveTimeout" which you can configure through session configuration.
    8. CONNECTED: when a user reconnect their lobby websocket, their status will change from DISCONNECTED to CONNECTED, only if they previously JOINED session. if they were on INVITED state before the disconnect happened, the user's status will return back to INVITED after they reconnect.
    9. DROPPED: when "inactiveTimeout" is due and user never re-establish their websocket connection, we will drop them from the session.
    10. TERMINATED: only applies to game session. If a game session (match) is ended, DS will end/delete the session and we will mark all remaining users' status to be TERMINATED.
    11. CANCELLED: when a session joinability changes to CLOSED, any remaining invites will be canceled.

    User is considered as active if they're present in the session, which their status either CONNECTED or JOINED.
    User has a reserved seat in the session if their status either INVITED, JOINED, CONNECTED, DISCONNECTED. When user's' status change to other than these mentioned statuses, we will release the seat for other players to occupy.

    Managing the relation between session and lobby websocket connection:
    - Session relies on lobby to consider player's connection health to our backend. therefore a disruption to lobby websocket will be reflected in the user's status in all of their session(s).
    - If user still have a reserved seat in the session and they disconnect lobby websocket, their status in session(s) will be changed to DISCONNECTED and field "members.previousStatus" of that user will contains the initial status before they disconnect lobby websocket. This "members.previousStatus" used to track user's previous status before they disconnect websocket, since we still reserve a seat for them, therefore this field will be empty again after they websocket.
    - If the disconnected user is the leader of the session they're disconnected from, we will wait until "leaderElectionGracePeriod" is due, to promote the next oldest member as the new leader of the session. You can configure "leaderElectionGracePeriod" through session configuration.
    - The user will be given chance to reconnect within "inactiveTimeout" which you can configure through session configuration. If until "inactiveTimeout" is due and the user doesn't reconnect their websocket, they will be removed from session and their status will change to DROPPED. If the dropped user was the leader of the session, we will promote the next oldest member as leader.
    - By default, we will update user's status to what it was before disconnect, when the user reconnects lobby websocket, unless "manualRejoin" sets to True in the session configuration. When "manualRejoin" is enabled, after lobby websocket reconnect, the game client will need to manually invoke join session again to rejoin the session.
    - If the user was on INVITED state before the disconnect happened, the user's status will return back to INVITED after they reconnect.
    - ttlHours is how long the session will active, max value is 168 hours

    Properties:
        url: /session/v1/public/namespaces/{namespace}/party

        method: POST

        tags: ["Party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsCreatePartyRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsPartySessionResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """

    # region fields

    _url: str = "/session/v1/public/namespaces/{namespace}/party"
    _method: str = "POST"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    body: ApimodelsCreatePartyRequest  # REQUIRED in [body]
    namespace: str  # REQUIRED in [path]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
            "path": self.get_path_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return self.body.to_dict()

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(self, value: ApimodelsCreatePartyRequest) -> PublicCreateParty:
        self.body = value
        return self

    def with_namespace(self, value: str) -> PublicCreateParty:
        self.namespace = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = self.body.to_dict(include_empty=include_empty)
        elif include_empty:
            result["body"] = ApimodelsCreatePartyRequest()
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, ApimodelsPartySessionResponse],
        Union[None, HttpResponse, ResponseError],
    ]:
        """Parse the given response.

        200: OK - ApimodelsPartySessionResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return ApimodelsPartySessionResponse.create_from_dict(content), None
        if code == 400:
            return None, ResponseError.create_from_dict(content)
        if code == 401:
            return None, ResponseError.create_from_dict(content)
        if code == 500:
            return None, ResponseError.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls, body: ApimodelsCreatePartyRequest, namespace: str, **kwargs
    ) -> PublicCreateParty:
        instance = cls()
        instance.body = body
        instance.namespace = namespace
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PublicCreateParty:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = ApimodelsCreatePartyRequest.create_from_dict(
                dict_["body"], include_empty=include_empty
            )
        elif include_empty:
            instance.body = ApimodelsCreatePartyRequest()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
            "namespace": "namespace",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": True,
            "namespace": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsListTagsResponse
from ..models import ModelsResponseError
from ..models import ModelsTagRequest

from ..operations.admin_tags import AdminDeleteTagHandlerV1
from ..operations.admin_tags import AdminListTagsHandlerV1
from ..operations.admin_tags import AdminPostTagHandlerV1


@same_doc_as(AdminDeleteTagHandlerV1)
def admin_delete_tag_handler_v1(
    tag: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a tag (adminDeleteTagHandlerV1)

    ## Description

    This endpoint will delete tag by name

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/tags/{tag}

        method: DELETE

        tags: ["AdminTags"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        tag: (tag) REQUIRED str in path

    Responses:
        204: No Content - (Tag deleted)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18510: tag not found)

        500: Internal Server Error - ModelsResponseError (18509: unable to delete tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTagHandlerV1.create(
        tag=tag,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteTagHandlerV1)
async def admin_delete_tag_handler_v1_async(
    tag: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a tag (adminDeleteTagHandlerV1)

    ## Description

    This endpoint will delete tag by name

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/tags/{tag}

        method: DELETE

        tags: ["AdminTags"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        tag: (tag) REQUIRED str in path

    Responses:
        204: No Content - (Tag deleted)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18510: tag not found)

        500: Internal Server Error - ModelsResponseError (18509: unable to delete tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTagHandlerV1.create(
        tag=tag,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminListTagsHandlerV1)
def admin_list_tags_handler_v1(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List tags (adminListTagsHandlerV1)

    ## Description

    Retrieve list of available tags by namespace

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/tags

        method: GET

        tags: ["AdminTags"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelsListTagsResponse (Available tags retrieved)

        400: Bad Request - ModelsResponseError (18503: unable to list tags)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18502: unable to list tags)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminListTagsHandlerV1.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminListTagsHandlerV1)
async def admin_list_tags_handler_v1_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List tags (adminListTagsHandlerV1)

    ## Description

    Retrieve list of available tags by namespace

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/tags

        method: GET

        tags: ["AdminTags"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelsListTagsResponse (Available tags retrieved)

        400: Bad Request - ModelsResponseError (18503: unable to list tags)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18502: unable to list tags)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminListTagsHandlerV1.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPostTagHandlerV1)
def admin_post_tag_handler_v1(
    body: ModelsTagRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a tag (adminPostTagHandlerV1)

    ## Description

    This endpoint will create new tags

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/tags

        method: POST

        tags: ["AdminTags"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsTagRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - (Tag created)

        400: Bad Request - ModelsResponseError (18505: invalid request body | 20002: validation error)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18506: tag already exists)

        500: Internal Server Error - ModelsResponseError (20000: internal server error | 18507: unable to create tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPostTagHandlerV1.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPostTagHandlerV1)
async def admin_post_tag_handler_v1_async(
    body: ModelsTagRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a tag (adminPostTagHandlerV1)

    ## Description

    This endpoint will create new tags

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/tags

        method: POST

        tags: ["AdminTags"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsTagRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - (Tag created)

        400: Bad Request - ModelsResponseError (18505: invalid request body | 20002: validation error)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18506: tag already exists)

        500: Internal Server Error - ModelsResponseError (20000: internal server error | 18507: unable to create tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPostTagHandlerV1.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

import functools
import warnings

from typing import Callable, Optional


def deprecated(
    func: Optional[Callable] = None,
    message: Optional[str] = None,
    replacement: Optional[str] = None,
):
    def decorator(f: Callable):
        @functools.wraps(f)
        def new_func(*args, **kwargs):
            text = f"Call to deprecated function '{f.__name__}'."
            if message:
                text += f" {message}"
            if replacement:
                text += f"Call `{replacement}` instead."
            warnings.simplefilter("always", DeprecationWarning)
            warnings.warn(
                text,
                category=DeprecationWarning,
                stacklevel=2,
            )
            warnings.simplefilter("default", DeprecationWarning)
            return f(*args, **kwargs)

        return new_func

    # case: @deprecated used directly
    if callable(func):
        return decorator(func)

    # case: @deprecated(...) used with arguments
    return decorator


def same_doc_as(source):
    def wrapper(target):
        target.__doc__ = source.__doc__
        return target

    return wrapper



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Fleet Commander

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApiTimeout(Model):
    """Contains fleet timeout configuration.  Fields accept duration strings (e.g. '30s', '1h30m'), an integer number of seconds. (api.Timeout)

    Properties:
        claim: (claim) OPTIONAL int

        creation: (creation) OPTIONAL int

        drain: (drain) OPTIONAL int

        session: (session) OPTIONAL int

        unresponsive: (unresponsive) OPTIONAL int
    """

    # region fields

    claim: int  # OPTIONAL
    creation: int  # OPTIONAL
    drain: int  # OPTIONAL
    session: int  # OPTIONAL
    unresponsive: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_claim(self, value: int) -> ApiTimeout:
        self.claim = value
        return self

    def with_creation(self, value: int) -> ApiTimeout:
        self.creation = value
        return self

    def with_drain(self, value: int) -> ApiTimeout:
        self.drain = value
        return self

    def with_session(self, value: int) -> ApiTimeout:
        self.session = value
        return self

    def with_unresponsive(self, value: int) -> ApiTimeout:
        self.unresponsive = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "claim"):
            result["claim"] = int(self.claim)
        elif include_empty:
            result["claim"] = 0
        if hasattr(self, "creation"):
            result["creation"] = int(self.creation)
        elif include_empty:
            result["creation"] = 0
        if hasattr(self, "drain"):
            result["drain"] = int(self.drain)
        elif include_empty:
            result["drain"] = 0
        if hasattr(self, "session"):
            result["session"] = int(self.session)
        elif include_empty:
            result["session"] = 0
        if hasattr(self, "unresponsive"):
            result["unresponsive"] = int(self.unresponsive)
        elif include_empty:
            result["unresponsive"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        claim: Optional[int] = None,
        creation: Optional[int] = None,
        drain: Optional[int] = None,
        session: Optional[int] = None,
        unresponsive: Optional[int] = None,
        **kwargs,
    ) -> ApiTimeout:
        instance = cls()
        if claim is not None:
            instance.claim = claim
        if creation is not None:
            instance.creation = creation
        if drain is not None:
            instance.drain = drain
        if session is not None:
            instance.session = session
        if unresponsive is not None:
            instance.unresponsive = unresponsive
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> ApiTimeout:
        instance = cls()
        if not dict_:
            return instance
        if "claim" in dict_ and dict_["claim"] is not None:
            instance.claim = int(dict_["claim"])
        elif include_empty:
            instance.claim = 0
        if "creation" in dict_ and dict_["creation"] is not None:
            instance.creation = int(dict_["creation"])
        elif include_empty:
            instance.creation = 0
        if "drain" in dict_ and dict_["drain"] is not None:
            instance.drain = int(dict_["drain"])
        elif include_empty:
            instance.drain = 0
        if "session" in dict_ and dict_["session"] is not None:
            instance.session = int(dict_["session"])
        elif include_empty:
            instance.session = 0
        if "unresponsive" in dict_ and dict_["unresponsive"] is not None:
            instance.unresponsive = int(dict_["unresponsive"])
        elif include_empty:
            instance.unresponsive = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiTimeout]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiTimeout]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ApiTimeout, List[ApiTimeout], Dict[Any, ApiTimeout]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "claim": "claim",
            "creation": "creation",
            "drain": "drain",
            "session": "session",
            "unresponsive": "unresponsive",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "claim": False,
            "creation": False,
            "drain": False,
            "session": False,
            "unresponsive": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class ItemDeletionCheckConfigEnum(StrEnum):
    CAMPAIGN = "CAMPAIGN"
    CATALOG = "CATALOG"
    DLC = "DLC"
    ENTITLEMENT = "ENTITLEMENT"
    IAP = "IAP"
    REWARD = "REWARD"


class CatalogConfigInfo(Model):
    """Catalog config info (CatalogConfigInfo)

    Properties:
        enable_inventory_check: (enableInventoryCheck) REQUIRED bool

        item_deletion_check_config: (itemDeletionCheckConfig) OPTIONAL List[Union[str, ItemDeletionCheckConfigEnum]]
    """

    # region fields

    enable_inventory_check: bool  # REQUIRED
    item_deletion_check_config: List[
        Union[str, ItemDeletionCheckConfigEnum]
    ]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_enable_inventory_check(self, value: bool) -> CatalogConfigInfo:
        self.enable_inventory_check = value
        return self

    def with_item_deletion_check_config(
        self, value: List[Union[str, ItemDeletionCheckConfigEnum]]
    ) -> CatalogConfigInfo:
        self.item_deletion_check_config = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "enable_inventory_check"):
            result["enableInventoryCheck"] = bool(self.enable_inventory_check)
        elif include_empty:
            result["enableInventoryCheck"] = False
        if hasattr(self, "item_deletion_check_config"):
            result["itemDeletionCheckConfig"] = [
                str(i0) for i0 in self.item_deletion_check_config
            ]
        elif include_empty:
            result["itemDeletionCheckConfig"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        enable_inventory_check: bool,
        item_deletion_check_config: Optional[
            List[Union[str, ItemDeletionCheckConfigEnum]]
        ] = None,
        **kwargs,
    ) -> CatalogConfigInfo:
        instance = cls()
        instance.enable_inventory_check = enable_inventory_check
        if item_deletion_check_config is not None:
            instance.item_deletion_check_config = item_deletion_check_config
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> CatalogConfigInfo:
        instance = cls()
        if not dict_:
            return instance
        if (
            "enableInventoryCheck" in dict_
            and dict_["enableInventoryCheck"] is not None
        ):
            instance.enable_inventory_check = bool(dict_["enableInventoryCheck"])
        elif include_empty:
            instance.enable_inventory_check = False
        if (
            "itemDeletionCheckConfig" in dict_
            and dict_["itemDeletionCheckConfig"] is not None
        ):
            instance.item_deletion_check_config = [
                str(i0) for i0 in dict_["itemDeletionCheckConfig"]
            ]
        elif include_empty:
            instance.item_deletion_check_config = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, CatalogConfigInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[CatalogConfigInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        CatalogConfigInfo, List[CatalogConfigInfo], Dict[Any, CatalogConfigInfo]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "enableInventoryCheck": "enable_inventory_check",
            "itemDeletionCheckConfig": "item_deletion_check_config",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "enableInventoryCheck": True,
            "itemDeletionCheckConfig": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "itemDeletionCheckConfig": [
                "CAMPAIGN",
                "CATALOG",
                "DLC",
                "ENTITLEMENT",
                "IAP",
                "REWARD",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Match Service V2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsPlayFeatureFlag(Model):
    """Models play feature flag (models.PlayFeatureFlag)

    Properties:
        enable_auto_cancel_match_user_disconnect: (ENABLE_AUTO_CANCEL_MATCH_USER_DISCONNECT) REQUIRED bool

        enable_auto_cancel_match_user_leave: (ENABLE_AUTO_CANCEL_MATCH_USER_LEAVE) REQUIRED bool
    """

    # region fields

    enable_auto_cancel_match_user_disconnect: bool  # REQUIRED
    enable_auto_cancel_match_user_leave: bool  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_enable_auto_cancel_match_user_disconnect(
        self, value: bool
    ) -> ModelsPlayFeatureFlag:
        self.enable_auto_cancel_match_user_disconnect = value
        return self

    def with_enable_auto_cancel_match_user_leave(
        self, value: bool
    ) -> ModelsPlayFeatureFlag:
        self.enable_auto_cancel_match_user_leave = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "enable_auto_cancel_match_user_disconnect"):
            result["ENABLE_AUTO_CANCEL_MATCH_USER_DISCONNECT"] = bool(
                self.enable_auto_cancel_match_user_disconnect
            )
        elif include_empty:
            result["ENABLE_AUTO_CANCEL_MATCH_USER_DISCONNECT"] = False
        if hasattr(self, "enable_auto_cancel_match_user_leave"):
            result["ENABLE_AUTO_CANCEL_MATCH_USER_LEAVE"] = bool(
                self.enable_auto_cancel_match_user_leave
            )
        elif include_empty:
            result["ENABLE_AUTO_CANCEL_MATCH_USER_LEAVE"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        enable_auto_cancel_match_user_disconnect: bool,
        enable_auto_cancel_match_user_leave: bool,
        **kwargs,
    ) -> ModelsPlayFeatureFlag:
        instance = cls()
        instance.enable_auto_cancel_match_user_disconnect = (
            enable_auto_cancel_match_user_disconnect
        )
        instance.enable_auto_cancel_match_user_leave = (
            enable_auto_cancel_match_user_leave
        )
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsPlayFeatureFlag:
        instance = cls()
        if not dict_:
            return instance
        if (
            "ENABLE_AUTO_CANCEL_MATCH_USER_DISCONNECT" in dict_
            and dict_["ENABLE_AUTO_CANCEL_MATCH_USER_DISCONNECT"] is not None
        ):
            instance.enable_auto_cancel_match_user_disconnect = bool(
                dict_["ENABLE_AUTO_CANCEL_MATCH_USER_DISCONNECT"]
            )
        elif include_empty:
            instance.enable_auto_cancel_match_user_disconnect = False
        if (
            "ENABLE_AUTO_CANCEL_MATCH_USER_LEAVE" in dict_
            and dict_["ENABLE_AUTO_CANCEL_MATCH_USER_LEAVE"] is not None
        ):
            instance.enable_auto_cancel_match_user_leave = bool(
                dict_["ENABLE_AUTO_CANCEL_MATCH_USER_LEAVE"]
            )
        elif include_empty:
            instance.enable_auto_cancel_match_user_leave = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsPlayFeatureFlag]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsPlayFeatureFlag]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsPlayFeatureFlag,
        List[ModelsPlayFeatureFlag],
        Dict[Any, ModelsPlayFeatureFlag],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "ENABLE_AUTO_CANCEL_MATCH_USER_DISCONNECT": "enable_auto_cancel_match_user_disconnect",
            "ENABLE_AUTO_CANCEL_MATCH_USER_LEAVE": "enable_auto_cancel_match_user_leave",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "ENABLE_AUTO_CANCEL_MATCH_USER_DISCONNECT": True,
            "ENABLE_AUTO_CANCEL_MATCH_USER_LEAVE": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Leaderboard Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.leaderboard.models import ModelsArchiveLeaderboardReq
from ..api.leaderboard.models import ModelsArchiveLeaderboardSignedURLResponse
from ..api.leaderboard.models import ModelsBulkUserIDsRequest
from ..api.leaderboard.models import ModelsBulkUserRankingResponseV3
from ..api.leaderboard.models import ModelsCycleInfo
from ..api.leaderboard.models import ModelsDailyConfig
from ..api.leaderboard.models import ModelsDeleteBulkLeaderboardFailedResp
from ..api.leaderboard.models import ModelsDeleteBulkLeaderboardSuccessResp
from ..api.leaderboard.models import ModelsDeleteBulkLeaderboardsReq
from ..api.leaderboard.models import ModelsDeleteBulkLeaderboardsResp
from ..api.leaderboard.models import ModelsGetAllLeaderboardConfigsPublicResp
from ..api.leaderboard.models import ModelsGetAllLeaderboardConfigsPublicRespV3
from ..api.leaderboard.models import ModelsGetAllLeaderboardConfigsResp
from ..api.leaderboard.models import ModelsGetAllLeaderboardConfigsRespV3
from ..api.leaderboard.models import ModelsGetAllUserLeaderboardsResp
from ..api.leaderboard.models import ModelsGetAllUserLeaderboardsRespV3
from ..api.leaderboard.models import ModelsGetHiddenUserResponse
from ..api.leaderboard.models import ModelsGetLeaderboardConfigPublicResp
from ..api.leaderboard.models import ModelsGetLeaderboardConfigPublicRespV3
from ..api.leaderboard.models import ModelsGetLeaderboardConfigResp
from ..api.leaderboard.models import ModelsGetLeaderboardConfigRespV3
from ..api.leaderboard.models import ModelsGetLeaderboardRankingResp
from ..api.leaderboard.models import ModelsGetUserVisibilityResponse
from ..api.leaderboard.models import ModelsLeaderboardConfigReq
from ..api.leaderboard.models import ModelsLeaderboardConfigReqV3
from ..api.leaderboard.models import ModelsMonthlyConfig
from ..api.leaderboard.models import ModelsPagination
from ..api.leaderboard.models import ModelsPaginationV3
from ..api.leaderboard.models import ModelsSetUserVisibilityRequest
from ..api.leaderboard.models import ModelsUpdateLeaderboardConfigReq
from ..api.leaderboard.models import ModelsUpdateLeaderboardConfigReqV3
from ..api.leaderboard.models import ModelsUpdateUserPointAdminV1Request
from ..api.leaderboard.models import ModelsUpdateUserPointAdminV1Response
from ..api.leaderboard.models import ModelsUserCycleRankingResponseDetail
from ..api.leaderboard.models import ModelsUserLeaderboardRanking
from ..api.leaderboard.models import ModelsUserLeaderboardRankingV3
from ..api.leaderboard.models import ModelsUserPoint
from ..api.leaderboard.models import ModelsUserRankingResponse
from ..api.leaderboard.models import ModelsUserRankingResponseDetail
from ..api.leaderboard.models import ModelsUserRankingResponseDetailV3
from ..api.leaderboard.models import ModelsUserRankingResponseV3
from ..api.leaderboard.models import ModelsWeeklyConfig
from ..api.leaderboard.models import ResponseErrorResponse
from ..api.leaderboard.models import V2Entry
from ..api.leaderboard.models import V2GetAllLeaderboardConfigsPublicResp
from ..api.leaderboard.models import V2GetLeaderboardConfigPublicResp
from ..api.leaderboard.models import V2GetPublicLeaderboardRankingResponse
from ..api.leaderboard.models import V2PaginationV2


def create_models_archive_leaderboard_req_example() -> ModelsArchiveLeaderboardReq:
    instance = ModelsArchiveLeaderboardReq()
    instance.leaderboard_codes = [randomize()]
    instance.limit = randomize("int", min_val=1, max_val=1000)
    instance.slug = randomize()
    return instance


def create_models_archive_leaderboard_signed_url_response_example() -> (
    ModelsArchiveLeaderboardSignedURLResponse
):
    instance = ModelsArchiveLeaderboardSignedURLResponse()
    instance.leaderboard_code = randomize()
    instance.url = randomize("url")
    return instance


def create_models_bulk_user_i_ds_request_example() -> ModelsBulkUserIDsRequest:
    instance = ModelsBulkUserIDsRequest()
    instance.user_ids = [randomize()]
    return instance


def create_models_bulk_user_ranking_response_v3_example() -> (
    ModelsBulkUserRankingResponseV3
):
    instance = ModelsBulkUserRankingResponseV3()
    instance.data = [create_models_user_ranking_response_v3_example()]
    return instance


def create_models_cycle_info_example() -> ModelsCycleInfo:
    instance = ModelsCycleInfo()
    instance.version = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_daily_config_example() -> ModelsDailyConfig:
    instance = ModelsDailyConfig()
    instance.reset_time = randomize()
    return instance


def create_models_delete_bulk_leaderboard_failed_resp_example() -> (
    ModelsDeleteBulkLeaderboardFailedResp
):
    instance = ModelsDeleteBulkLeaderboardFailedResp()
    instance.error = randomize()
    instance.leaderboard_code = randomize()
    return instance


def create_models_delete_bulk_leaderboard_success_resp_example() -> (
    ModelsDeleteBulkLeaderboardSuccessResp
):
    instance = ModelsDeleteBulkLeaderboardSuccessResp()
    instance.leaderboard_code = randomize()
    return instance


def create_models_delete_bulk_leaderboards_req_example() -> (
    ModelsDeleteBulkLeaderboardsReq
):
    instance = ModelsDeleteBulkLeaderboardsReq()
    instance.leaderboard_codes = [randomize()]
    return instance


def create_models_delete_bulk_leaderboards_resp_example() -> (
    ModelsDeleteBulkLeaderboardsResp
):
    instance = ModelsDeleteBulkLeaderboardsResp()
    instance.failed = [create_models_delete_bulk_leaderboard_failed_resp_example()]
    instance.success = [create_models_delete_bulk_leaderboard_success_resp_example()]
    return instance


def create_models_get_all_leaderboard_configs_public_resp_example() -> (
    ModelsGetAllLeaderboardConfigsPublicResp
):
    instance = ModelsGetAllLeaderboardConfigsPublicResp()
    instance.data = [create_models_get_leaderboard_config_public_resp_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_get_all_leaderboard_configs_public_resp_v3_example() -> (
    ModelsGetAllLeaderboardConfigsPublicRespV3
):
    instance = ModelsGetAllLeaderboardConfigsPublicRespV3()
    instance.data = [create_models_get_leaderboard_config_public_resp_v3_example()]
    instance.paging = create_models_pagination_v3_example()
    return instance


def create_models_get_all_leaderboard_configs_resp_example() -> (
    ModelsGetAllLeaderboardConfigsResp
):
    instance = ModelsGetAllLeaderboardConfigsResp()
    instance.data = [create_models_get_leaderboard_config_resp_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_get_all_leaderboard_configs_resp_v3_example() -> (
    ModelsGetAllLeaderboardConfigsRespV3
):
    instance = ModelsGetAllLeaderboardConfigsRespV3()
    instance.data = [create_models_get_leaderboard_config_resp_v3_example()]
    instance.paging = create_models_pagination_v3_example()
    return instance


def create_models_get_all_user_leaderboards_resp_example() -> (
    ModelsGetAllUserLeaderboardsResp
):
    instance = ModelsGetAllUserLeaderboardsResp()
    instance.data = [create_models_user_leaderboard_ranking_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_get_all_user_leaderboards_resp_v3_example() -> (
    ModelsGetAllUserLeaderboardsRespV3
):
    instance = ModelsGetAllUserLeaderboardsRespV3()
    instance.data = [create_models_user_leaderboard_ranking_v3_example()]
    instance.paging = create_models_pagination_v3_example()
    return instance


def create_models_get_hidden_user_response_example() -> ModelsGetHiddenUserResponse:
    instance = ModelsGetHiddenUserResponse()
    instance.data = [randomize()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_get_leaderboard_config_public_resp_example() -> (
    ModelsGetLeaderboardConfigPublicResp
):
    instance = ModelsGetLeaderboardConfigPublicResp()
    instance.icon_url = randomize("url")
    instance.leaderboard_code = randomize()
    instance.name = randomize()
    instance.stat_code = randomize()
    return instance


def create_models_get_leaderboard_config_public_resp_v3_example() -> (
    ModelsGetLeaderboardConfigPublicRespV3
):
    instance = ModelsGetLeaderboardConfigPublicRespV3()
    instance.all_time = randomize("bool")
    instance.cycle_ids = [randomize()]
    instance.icon_url = randomize("url")
    instance.leaderboard_code = randomize()
    instance.name = randomize()
    instance.stat_code = randomize()
    instance.cycle_metadata = {}
    return instance


def create_models_get_leaderboard_config_resp_example() -> (
    ModelsGetLeaderboardConfigResp
):
    instance = ModelsGetLeaderboardConfigResp()
    instance.daily = create_models_daily_config_example()
    instance.descending = randomize("bool")
    instance.icon_url = randomize("url")
    instance.is_archived = randomize("bool")
    instance.is_deleted = randomize("bool")
    instance.leaderboard_code = randomize()
    instance.monthly = create_models_monthly_config_example()
    instance.name = randomize()
    instance.season_period = randomize("int", min_val=1, max_val=1000)
    instance.start_time = randomize()
    instance.stat_code = randomize()
    instance.weekly = create_models_weekly_config_example()
    instance.deleted_at = randomize("date")
    return instance


def create_models_get_leaderboard_config_resp_v3_example() -> (
    ModelsGetLeaderboardConfigRespV3
):
    instance = ModelsGetLeaderboardConfigRespV3()
    instance.all_time = randomize("bool")
    instance.created_at = randomize("date")
    instance.cycle_ids = [randomize()]
    instance.descending = randomize("bool")
    instance.enable_server_hidden_filter = randomize("bool")
    instance.is_deleted = randomize("bool")
    instance.leaderboard_code = randomize()
    instance.name = randomize()
    instance.stat_code = randomize()
    instance.updated_at = randomize("date")
    instance.cycle_metadata = {}
    instance.deleted_at = randomize("date")
    instance.description = randomize()
    instance.icon_url = randomize("url")
    return instance


def create_models_get_leaderboard_ranking_resp_example() -> (
    ModelsGetLeaderboardRankingResp
):
    instance = ModelsGetLeaderboardRankingResp()
    instance.data = [create_models_user_point_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_get_user_visibility_response_example() -> (
    ModelsGetUserVisibilityResponse
):
    instance = ModelsGetUserVisibilityResponse()
    instance.namespace = randomize("slug")
    instance.user_id = randomize("uid")
    instance.visibility = randomize("bool")
    return instance


def create_models_leaderboard_config_req_example() -> ModelsLeaderboardConfigReq:
    instance = ModelsLeaderboardConfigReq()
    instance.daily = create_models_daily_config_example()
    instance.descending = randomize("bool")
    instance.icon_url = randomize("url")
    instance.leaderboard_code = randomize()
    instance.monthly = create_models_monthly_config_example()
    instance.name = randomize()
    instance.season_period = randomize("int", min_val=1, max_val=1000)
    instance.start_time = randomize()
    instance.stat_code = randomize()
    instance.weekly = create_models_weekly_config_example()
    return instance


def create_models_leaderboard_config_req_v3_example() -> ModelsLeaderboardConfigReqV3:
    instance = ModelsLeaderboardConfigReqV3()
    instance.all_time = randomize("bool")
    instance.descending = randomize("bool")
    instance.leaderboard_code = randomize()
    instance.name = randomize()
    instance.stat_code = randomize()
    instance.cycle_ids = [randomize()]
    instance.description = randomize()
    instance.enable_server_hidden_filter = randomize("bool")
    instance.icon_url = randomize("url")
    return instance


def create_models_monthly_config_example() -> ModelsMonthlyConfig:
    instance = ModelsMonthlyConfig()
    instance.reset_date = randomize("int", min_val=1, max_val=1000)
    instance.reset_time = randomize()
    return instance


def create_models_pagination_example() -> ModelsPagination:
    instance = ModelsPagination()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_models_pagination_v3_example() -> ModelsPaginationV3:
    instance = ModelsPaginationV3()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_models_set_user_visibility_request_example() -> (
    ModelsSetUserVisibilityRequest
):
    instance = ModelsSetUserVisibilityRequest()
    instance.visibility = randomize("bool")
    return instance


def create_models_update_leaderboard_config_req_example() -> (
    ModelsUpdateLeaderboardConfigReq
):
    instance = ModelsUpdateLeaderboardConfigReq()
    instance.daily = create_models_daily_config_example()
    instance.descending = randomize("bool")
    instance.icon_url = randomize("url")
    instance.monthly = create_models_monthly_config_example()
    instance.name = randomize()
    instance.season_period = randomize("int", min_val=1, max_val=1000)
    instance.start_time = randomize()
    instance.stat_code = randomize()
    instance.weekly = create_models_weekly_config_example()
    return instance


def create_models_update_leaderboard_config_req_v3_example() -> (
    ModelsUpdateLeaderboardConfigReqV3
):
    instance = ModelsUpdateLeaderboardConfigReqV3()
    instance.all_time = randomize("bool")
    instance.descending = randomize("bool")
    instance.name = randomize()
    instance.cycle_ids = [randomize()]
    instance.description = randomize()
    instance.enable_server_hidden_filter = randomize("bool")
    instance.icon_url = randomize("url")
    return instance


def create_models_update_user_point_admin_v1_request_example() -> (
    ModelsUpdateUserPointAdminV1Request
):
    instance = ModelsUpdateUserPointAdminV1Request()
    instance.inc = randomize("int", min_val=1, max_val=1000)
    instance.latest_value = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_update_user_point_admin_v1_response_example() -> (
    ModelsUpdateUserPointAdminV1Response
):
    instance = ModelsUpdateUserPointAdminV1Response()
    instance.point = randomize("int", min_val=1, max_val=1000)
    instance.user_id = randomize("uid")
    return instance


def create_models_user_cycle_ranking_response_detail_example() -> (
    ModelsUserCycleRankingResponseDetail
):
    instance = ModelsUserCycleRankingResponseDetail()
    instance.cycle_id = randomize()
    instance.point = randomize("int", min_val=1, max_val=1000)
    instance.rank = randomize("int", min_val=1, max_val=1000)
    instance.additional_data = {randomize(): randomize()}
    instance.hidden = randomize("bool")
    return instance


def create_models_user_leaderboard_ranking_example() -> ModelsUserLeaderboardRanking:
    instance = ModelsUserLeaderboardRanking()
    instance.all_time = create_models_user_ranking_response_detail_example()
    instance.current = create_models_user_ranking_response_detail_example()
    instance.daily = create_models_user_ranking_response_detail_example()
    instance.leaderboard_code = randomize()
    instance.leaderboard_name = randomize()
    instance.monthly = create_models_user_ranking_response_detail_example()
    instance.user_id = randomize("uid")
    instance.weekly = create_models_user_ranking_response_detail_example()
    return instance


def create_models_user_leaderboard_ranking_v3_example() -> (
    ModelsUserLeaderboardRankingV3
):
    instance = ModelsUserLeaderboardRankingV3()
    instance.cycles = [create_models_user_cycle_ranking_response_detail_example()]
    instance.leaderboard_code = randomize()
    instance.leaderboard_name = randomize()
    instance.user_id = randomize("uid")
    instance.all_time = create_models_user_ranking_response_detail_v3_example()
    return instance


def create_models_user_point_example() -> ModelsUserPoint:
    instance = ModelsUserPoint()
    instance.additional_data = {randomize(): randomize()}
    instance.point = randomize("int", min_val=1, max_val=1000)
    instance.user_id = randomize("uid")
    instance.hidden = randomize("bool")
    return instance


def create_models_user_ranking_response_example() -> ModelsUserRankingResponse:
    instance = ModelsUserRankingResponse()
    instance.all_time = create_models_user_ranking_response_detail_example()
    instance.current = create_models_user_ranking_response_detail_example()
    instance.daily = create_models_user_ranking_response_detail_example()
    instance.monthly = create_models_user_ranking_response_detail_example()
    instance.user_id = randomize("uid")
    instance.weekly = create_models_user_ranking_response_detail_example()
    return instance


def create_models_user_ranking_response_detail_example() -> (
    ModelsUserRankingResponseDetail
):
    instance = ModelsUserRankingResponseDetail()
    instance.additional_data = {randomize(): randomize()}
    instance.point = randomize("int", min_val=1, max_val=1000)
    instance.rank = randomize("int", min_val=1, max_val=1000)
    instance.hidden = randomize("bool")
    return instance


def create_models_user_ranking_response_detail_v3_example() -> (
    ModelsUserRankingResponseDetailV3
):
    instance = ModelsUserRankingResponseDetailV3()
    instance.point = randomize("int", min_val=1, max_val=1000)
    instance.rank = randomize("int", min_val=1, max_val=1000)
    instance.additional_data = {randomize(): randomize()}
    instance.hidden = randomize("bool")
    return instance


def create_models_user_ranking_response_v3_example() -> ModelsUserRankingResponseV3:
    instance = ModelsUserRankingResponseV3()
    instance.cycles = [create_models_user_cycle_ranking_response_detail_example()]
    instance.user_id = randomize("uid")
    instance.all_time = create_models_user_ranking_response_detail_v3_example()
    return instance


def create_models_weekly_config_example() -> ModelsWeeklyConfig:
    instance = ModelsWeeklyConfig()
    instance.reset_day = randomize("int", min_val=1, max_val=1000)
    instance.reset_time = randomize()
    return instance


def create_response_error_response_example() -> ResponseErrorResponse:
    instance = ResponseErrorResponse()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance


def create_v2_entry_example() -> V2Entry:
    instance = V2Entry()
    instance.point = randomize("int", min_val=1, max_val=1000)
    instance.additional_data = {randomize(): randomize()}
    instance.hidden = randomize("bool")
    return instance


def create_v2_get_all_leaderboard_configs_public_resp_example() -> (
    V2GetAllLeaderboardConfigsPublicResp
):
    instance = V2GetAllLeaderboardConfigsPublicResp()
    instance.data = [create_v2_get_leaderboard_config_public_resp_example()]
    instance.paging = create_v2_pagination_v2_example()
    return instance


def create_v2_get_leaderboard_config_public_resp_example() -> (
    V2GetLeaderboardConfigPublicResp
):
    instance = V2GetLeaderboardConfigPublicResp()
    instance.icon_url = randomize("url")
    instance.leaderboard_code = randomize()
    instance.name = randomize()
    instance.stat_code = randomize()
    return instance


def create_v2_get_public_leaderboard_ranking_response_example() -> (
    V2GetPublicLeaderboardRankingResponse
):
    instance = V2GetPublicLeaderboardRankingResponse()
    instance.data = [create_v2_entry_example()]
    instance.paging = create_v2_pagination_v2_example()
    return instance


def create_v2_pagination_v2_example() -> V2PaginationV2:
    instance = V2PaginationV2()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsPlayFeatureFlag
from ..models import ResponseError

from ..operations.play_feature_flag import AdminDeletePlayFeatureFlag
from ..operations.play_feature_flag import AdminGetPlayFeatureFlag
from ..operations.play_feature_flag import AdminUpsertPlayFeatureFlag


@same_doc_as(AdminDeletePlayFeatureFlag)
def admin_delete_play_feature_flag(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin Delete Play Feature Flag (adminDeletePlayFeatureFlag)


    Delete matchmaking Play Feature Flag.
    Deleting the current namespace's custom config only removes it from the database, the system will then use the environment variable defined in the ConfigMap instead.

    Properties:
        url: /match2/v1/admin/namespaces/{namespace}/playfeatureflag

        method: DELETE

        tags: ["PlayFeatureFlag", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeletePlayFeatureFlag.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeletePlayFeatureFlag)
async def admin_delete_play_feature_flag_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin Delete Play Feature Flag (adminDeletePlayFeatureFlag)


    Delete matchmaking Play Feature Flag.
    Deleting the current namespace's custom config only removes it from the database, the system will then use the environment variable defined in the ConfigMap instead.

    Properties:
        url: /match2/v1/admin/namespaces/{namespace}/playfeatureflag

        method: DELETE

        tags: ["PlayFeatureFlag", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeletePlayFeatureFlag.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetPlayFeatureFlag)
def admin_get_play_feature_flag(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin Play Feature Flag (adminGetPlayFeatureFlag)

    Get matchmaking Play Feature Flag.

    Properties:
        url: /match2/v1/admin/namespaces/{namespace}/playfeatureflag

        method: GET

        tags: ["PlayFeatureFlag", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayFeatureFlag (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetPlayFeatureFlag.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetPlayFeatureFlag)
async def admin_get_play_feature_flag_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin Play Feature Flag (adminGetPlayFeatureFlag)

    Get matchmaking Play Feature Flag.

    Properties:
        url: /match2/v1/admin/namespaces/{namespace}/playfeatureflag

        method: GET

        tags: ["PlayFeatureFlag", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayFeatureFlag (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetPlayFeatureFlag.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpsertPlayFeatureFlag)
def admin_upsert_play_feature_flag(
    body: ModelsPlayFeatureFlag,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin Upsert Play Feature Flag (adminUpsertPlayFeatureFlag)

    Upsert matchmaking Play Feature Flag.

    Properties:
        url: /match2/v1/admin/namespaces/{namespace}/playfeatureflag

        method: POST

        tags: ["PlayFeatureFlag", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPlayFeatureFlag in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayFeatureFlag (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpsertPlayFeatureFlag.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpsertPlayFeatureFlag)
async def admin_upsert_play_feature_flag_async(
    body: ModelsPlayFeatureFlag,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin Upsert Play Feature Flag (adminUpsertPlayFeatureFlag)

    Upsert matchmaking Play Feature Flag.

    Properties:
        url: /match2/v1/admin/namespaces/{namespace}/playfeatureflag

        method: POST

        tags: ["PlayFeatureFlag", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPlayFeatureFlag in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayFeatureFlag (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpsertPlayFeatureFlag.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelIntegrationListItem(Model):
    """Apimodel integration list item (apimodel.IntegrationListItem)

    Properties:
        app_id: (appId) REQUIRED str

        app_name: (appName) REQUIRED str

        created_at: (createdAt) REQUIRED str

        game_namespace: (gameNamespace) REQUIRED str

        integration_id: (integrationId) REQUIRED str

        resource_id: (resourceId) REQUIRED str

        scenario: (scenario) REQUIRED str

        status: (status) REQUIRED str

        status_message: (statusMessage) REQUIRED str

        studio_name: (studioName) REQUIRED str
    """

    # region fields

    app_id: str  # REQUIRED
    app_name: str  # REQUIRED
    created_at: str  # REQUIRED
    game_namespace: str  # REQUIRED
    integration_id: str  # REQUIRED
    resource_id: str  # REQUIRED
    scenario: str  # REQUIRED
    status: str  # REQUIRED
    status_message: str  # REQUIRED
    studio_name: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_app_id(self, value: str) -> ApimodelIntegrationListItem:
        self.app_id = value
        return self

    def with_app_name(self, value: str) -> ApimodelIntegrationListItem:
        self.app_name = value
        return self

    def with_created_at(self, value: str) -> ApimodelIntegrationListItem:
        self.created_at = value
        return self

    def with_game_namespace(self, value: str) -> ApimodelIntegrationListItem:
        self.game_namespace = value
        return self

    def with_integration_id(self, value: str) -> ApimodelIntegrationListItem:
        self.integration_id = value
        return self

    def with_resource_id(self, value: str) -> ApimodelIntegrationListItem:
        self.resource_id = value
        return self

    def with_scenario(self, value: str) -> ApimodelIntegrationListItem:
        self.scenario = value
        return self

    def with_status(self, value: str) -> ApimodelIntegrationListItem:
        self.status = value
        return self

    def with_status_message(self, value: str) -> ApimodelIntegrationListItem:
        self.status_message = value
        return self

    def with_studio_name(self, value: str) -> ApimodelIntegrationListItem:
        self.studio_name = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "app_id"):
            result["appId"] = str(self.app_id)
        elif include_empty:
            result["appId"] = ""
        if hasattr(self, "app_name"):
            result["appName"] = str(self.app_name)
        elif include_empty:
            result["appName"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "game_namespace"):
            result["gameNamespace"] = str(self.game_namespace)
        elif include_empty:
            result["gameNamespace"] = ""
        if hasattr(self, "integration_id"):
            result["integrationId"] = str(self.integration_id)
        elif include_empty:
            result["integrationId"] = ""
        if hasattr(self, "resource_id"):
            result["resourceId"] = str(self.resource_id)
        elif include_empty:
            result["resourceId"] = ""
        if hasattr(self, "scenario"):
            result["scenario"] = str(self.scenario)
        elif include_empty:
            result["scenario"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = ""
        if hasattr(self, "status_message"):
            result["statusMessage"] = str(self.status_message)
        elif include_empty:
            result["statusMessage"] = ""
        if hasattr(self, "studio_name"):
            result["studioName"] = str(self.studio_name)
        elif include_empty:
            result["studioName"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        app_id: str,
        app_name: str,
        created_at: str,
        game_namespace: str,
        integration_id: str,
        resource_id: str,
        scenario: str,
        status: str,
        status_message: str,
        studio_name: str,
        **kwargs,
    ) -> ApimodelIntegrationListItem:
        instance = cls()
        instance.app_id = app_id
        instance.app_name = app_name
        instance.created_at = created_at
        instance.game_namespace = game_namespace
        instance.integration_id = integration_id
        instance.resource_id = resource_id
        instance.scenario = scenario
        instance.status = status
        instance.status_message = status_message
        instance.studio_name = studio_name
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelIntegrationListItem:
        instance = cls()
        if not dict_:
            return instance
        if "appId" in dict_ and dict_["appId"] is not None:
            instance.app_id = str(dict_["appId"])
        elif include_empty:
            instance.app_id = ""
        if "appName" in dict_ and dict_["appName"] is not None:
            instance.app_name = str(dict_["appName"])
        elif include_empty:
            instance.app_name = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "gameNamespace" in dict_ and dict_["gameNamespace"] is not None:
            instance.game_namespace = str(dict_["gameNamespace"])
        elif include_empty:
            instance.game_namespace = ""
        if "integrationId" in dict_ and dict_["integrationId"] is not None:
            instance.integration_id = str(dict_["integrationId"])
        elif include_empty:
            instance.integration_id = ""
        if "resourceId" in dict_ and dict_["resourceId"] is not None:
            instance.resource_id = str(dict_["resourceId"])
        elif include_empty:
            instance.resource_id = ""
        if "scenario" in dict_ and dict_["scenario"] is not None:
            instance.scenario = str(dict_["scenario"])
        elif include_empty:
            instance.scenario = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = ""
        if "statusMessage" in dict_ and dict_["statusMessage"] is not None:
            instance.status_message = str(dict_["statusMessage"])
        elif include_empty:
            instance.status_message = ""
        if "studioName" in dict_ and dict_["studioName"] is not None:
            instance.studio_name = str(dict_["studioName"])
        elif include_empty:
            instance.studio_name = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelIntegrationListItem]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelIntegrationListItem]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelIntegrationListItem,
        List[ApimodelIntegrationListItem],
        Dict[Any, ApimodelIntegrationListItem],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "appId": "app_id",
            "appName": "app_name",
            "createdAt": "created_at",
            "gameNamespace": "game_namespace",
            "integrationId": "integration_id",
            "resourceId": "resource_id",
            "scenario": "scenario",
            "status": "status",
            "statusMessage": "status_message",
            "studioName": "studio_name",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "appId": True,
            "appName": True,
            "createdAt": True,
            "gameNamespace": True,
            "integrationId": True,
            "resourceId": True,
            "scenario": True,
            "status": True,
            "statusMessage": True,
            "studioName": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Fleet Commander

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.api_port_configuration import ApiPortConfiguration
from ..models.api_timeout import ApiTimeout


class ApiImageDeploymentProfile(Model):
    """Api image deployment profile (api.ImageDeploymentProfile)

    Properties:
        command_line: (commandLine) REQUIRED str

        image_id: (imageId) REQUIRED str

        port_configurations: (portConfigurations) REQUIRED List[ApiPortConfiguration]

        timeout: (timeout) OPTIONAL ApiTimeout
    """

    # region fields

    command_line: str  # REQUIRED
    image_id: str  # REQUIRED
    port_configurations: List[ApiPortConfiguration]  # REQUIRED
    timeout: ApiTimeout  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_command_line(self, value: str) -> ApiImageDeploymentProfile:
        self.command_line = value
        return self

    def with_image_id(self, value: str) -> ApiImageDeploymentProfile:
        self.image_id = value
        return self

    def with_port_configurations(
        self, value: List[ApiPortConfiguration]
    ) -> ApiImageDeploymentProfile:
        self.port_configurations = value
        return self

    def with_timeout(self, value: ApiTimeout) -> ApiImageDeploymentProfile:
        self.timeout = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "command_line"):
            result["commandLine"] = str(self.command_line)
        elif include_empty:
            result["commandLine"] = ""
        if hasattr(self, "image_id"):
            result["imageId"] = str(self.image_id)
        elif include_empty:
            result["imageId"] = ""
        if hasattr(self, "port_configurations"):
            result["portConfigurations"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.port_configurations
            ]
        elif include_empty:
            result["portConfigurations"] = []
        if hasattr(self, "timeout"):
            result["timeout"] = self.timeout.to_dict(include_empty=include_empty)
        elif include_empty:
            result["timeout"] = ApiTimeout()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        command_line: str,
        image_id: str,
        port_configurations: List[ApiPortConfiguration],
        timeout: Optional[ApiTimeout] = None,
        **kwargs,
    ) -> ApiImageDeploymentProfile:
        instance = cls()
        instance.command_line = command_line
        instance.image_id = image_id
        instance.port_configurations = port_configurations
        if timeout is not None:
            instance.timeout = timeout
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApiImageDeploymentProfile:
        instance = cls()
        if not dict_:
            return instance
        if "commandLine" in dict_ and dict_["commandLine"] is not None:
            instance.command_line = str(dict_["commandLine"])
        elif include_empty:
            instance.command_line = ""
        if "imageId" in dict_ and dict_["imageId"] is not None:
            instance.image_id = str(dict_["imageId"])
        elif include_empty:
            instance.image_id = ""
        if "portConfigurations" in dict_ and dict_["portConfigurations"] is not None:
            instance.port_configurations = [
                ApiPortConfiguration.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["portConfigurations"]
            ]
        elif include_empty:
            instance.port_configurations = []
        if "timeout" in dict_ and dict_["timeout"] is not None:
            instance.timeout = ApiTimeout.create_from_dict(
                dict_["timeout"], include_empty=include_empty
            )
        elif include_empty:
            instance.timeout = ApiTimeout()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiImageDeploymentProfile]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiImageDeploymentProfile]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApiImageDeploymentProfile,
        List[ApiImageDeploymentProfile],
        Dict[Any, ApiImageDeploymentProfile],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "commandLine": "command_line",
            "imageId": "image_id",
            "portConfigurations": "port_configurations",
            "timeout": "timeout",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "commandLine": True,
            "imageId": True,
            "portConfigurations": True,
            "timeout": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session History Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.sessionhistory.models import ApimodelsXRayBulkTicketObservabilityRequest
from ..api.sessionhistory.models import ApimodelsXRayBulkTicketObservabilityResponse
from ..api.sessionhistory.models import ApimodelsXRayTicketObservabilityRequest
from ..api.sessionhistory.models import ApimodelsXRayTicketObservabilityResponse
from ..api.sessionhistory.models import ModelsAllianceRule
from ..api.sessionhistory.models import ModelsMatchingRule
from ..api.sessionhistory.models import ResponseError


def create_apimodels_x_ray_bulk_ticket_observability_request_example() -> (
    ApimodelsXRayBulkTicketObservabilityRequest
):
    instance = ApimodelsXRayBulkTicketObservabilityRequest()
    instance.ticket_observabilities = [
        create_apimodels_x_ray_ticket_observability_request_example()
    ]
    return instance


def create_apimodels_x_ray_bulk_ticket_observability_response_example() -> (
    ApimodelsXRayBulkTicketObservabilityResponse
):
    instance = ApimodelsXRayBulkTicketObservabilityResponse()
    instance.ticket_i_ds = [randomize()]
    return instance


def create_apimodels_x_ray_ticket_observability_request_example() -> (
    ApimodelsXRayTicketObservabilityRequest
):
    instance = ApimodelsXRayTicketObservabilityRequest()
    instance.action = randomize()
    instance.game_mode = randomize()
    instance.is_backfill_match = randomize("bool")
    instance.is_rule_set_flexed = randomize("bool")
    instance.namespace = randomize("slug")
    instance.session_tick_id = randomize()
    instance.tick_id = randomize("int", min_val=1, max_val=1000)
    instance.ticket_id = randomize()
    instance.timestamp = randomize("date")
    instance.active_alliance_rule = create_models_alliance_rule_example()
    instance.active_matching_rule = [create_models_matching_rule_example()]
    instance.function = randomize()
    instance.iteration = randomize("int", min_val=1, max_val=1000)
    instance.match_id = randomize()
    instance.remaining_players_per_ticket = [randomize("int", min_val=1, max_val=1000)]
    instance.remaining_tickets = randomize("int", min_val=1, max_val=1000)
    instance.time_to_match_sec = randomize("int", min_val=1, max_val=1000)
    instance.unbackfill_reason = randomize()
    instance.unmatch_reason = randomize()
    return instance


def create_apimodels_x_ray_ticket_observability_response_example() -> (
    ApimodelsXRayTicketObservabilityResponse
):
    instance = ApimodelsXRayTicketObservabilityResponse()
    instance.ticket_id = randomize()
    return instance


def create_models_alliance_rule_example() -> ModelsAllianceRule:
    instance = ModelsAllianceRule()
    instance.max_number = randomize("int", min_val=1, max_val=1000)
    instance.min_number = randomize("int", min_val=1, max_val=1000)
    instance.player_max_number = randomize("int", min_val=1, max_val=1000)
    instance.player_min_number = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_matching_rule_example() -> ModelsMatchingRule:
    instance = ModelsMatchingRule()
    instance.attribute = randomize()
    instance.criteria = randomize()
    instance.reference = randomize("int", min_val=1, max_val=1000)
    return instance


def create_response_error_example() -> ResponseError:
    instance = ResponseError()
    instance.attributes = {randomize(): randomize()}
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    instance.message = randomize()
    instance.name = randomize()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelGetProfileUpdateStrategyConfigResponse
from ..models import ModelSimpleProfileUpdateStrategyConfigs
from ..models import ModelUpdateProfileUpdateStrategyConfigRequest
from ..models import RestErrorResponse

from ..operations.profile_update_strategy import AdminGetProfileUpdateStrategyV3
from ..operations.profile_update_strategy import (
    AdminGetProfileUpdateStrategyV3FieldEnum,
)
from ..operations.profile_update_strategy import AdminUpdateProfileUpdateStrategyV3
from ..operations.profile_update_strategy import (
    AdminUpdateProfileUpdateStrategyV3FieldEnum,
)
from ..operations.profile_update_strategy import PublicGetProfileUpdateStrategyV3
from ..operations.profile_update_strategy import (
    PublicGetProfileUpdateStrategyV3FieldEnum,
)


@same_doc_as(AdminGetProfileUpdateStrategyV3)
def admin_get_profile_update_strategy_v3(
    field: Optional[Union[str, AdminGetProfileUpdateStrategyV3FieldEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get profile update strategy by namespace and field. (AdminGetProfileUpdateStrategyV3)

    This API is for admin to get profile update strategy by namespace and field.
    Note: If the config is not found, this API will return a config with unlimited.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/profileUpdateStrategies

        method: GET

        tags: ["Profile Update Strategy"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        field: (field) OPTIONAL Union[str, FieldEnum] in query

    Responses:
        200: OK - ModelGetProfileUpdateStrategyConfigResponse (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetProfileUpdateStrategyV3.create(
        field=field,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetProfileUpdateStrategyV3)
async def admin_get_profile_update_strategy_v3_async(
    field: Optional[Union[str, AdminGetProfileUpdateStrategyV3FieldEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get profile update strategy by namespace and field. (AdminGetProfileUpdateStrategyV3)

    This API is for admin to get profile update strategy by namespace and field.
    Note: If the config is not found, this API will return a config with unlimited.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/profileUpdateStrategies

        method: GET

        tags: ["Profile Update Strategy"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        field: (field) OPTIONAL Union[str, FieldEnum] in query

    Responses:
        200: OK - ModelGetProfileUpdateStrategyConfigResponse (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetProfileUpdateStrategyV3.create(
        field=field,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateProfileUpdateStrategyV3)
def admin_update_profile_update_strategy_v3(
    body: ModelUpdateProfileUpdateStrategyConfigRequest,
    field: Union[str, AdminUpdateProfileUpdateStrategyV3FieldEnum],
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin update profile update strategy by namespace and field. (AdminUpdateProfileUpdateStrategyV3)

    Uses upsert behavior.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/profileUpdateStrategies

        method: PUT

        tags: ["Profile Update Strategy"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUpdateProfileUpdateStrategyConfigRequest in body

        namespace: (namespace) REQUIRED str in path

        field: (field) REQUIRED Union[str, FieldEnum] in query

    Responses:
        200: OK - ModelSimpleProfileUpdateStrategyConfigs (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateProfileUpdateStrategyV3.create(
        body=body,
        field=field,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateProfileUpdateStrategyV3)
async def admin_update_profile_update_strategy_v3_async(
    body: ModelUpdateProfileUpdateStrategyConfigRequest,
    field: Union[str, AdminUpdateProfileUpdateStrategyV3FieldEnum],
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin update profile update strategy by namespace and field. (AdminUpdateProfileUpdateStrategyV3)

    Uses upsert behavior.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/profileUpdateStrategies

        method: PUT

        tags: ["Profile Update Strategy"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUpdateProfileUpdateStrategyConfigRequest in body

        namespace: (namespace) REQUIRED str in path

        field: (field) REQUIRED Union[str, FieldEnum] in query

    Responses:
        200: OK - ModelSimpleProfileUpdateStrategyConfigs (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateProfileUpdateStrategyV3.create(
        body=body,
        field=field,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetProfileUpdateStrategyV3)
def public_get_profile_update_strategy_v3(
    field: Optional[Union[str, PublicGetProfileUpdateStrategyV3FieldEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Public get profile update strategy by namespace and field. (PublicGetProfileUpdateStrategyV3)

    This API is for public user to get profile update strategy by namespace and field.
    Note: If the config is not found, this API will return a config with unlimited.

    Properties:
        url: /iam/v3/public/namespaces/{namespace}/profileUpdateStrategies

        method: GET

        tags: ["Profile Update Strategy"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        field: (field) OPTIONAL Union[str, FieldEnum] in query

    Responses:
        200: OK - ModelGetProfileUpdateStrategyConfigResponse (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicGetProfileUpdateStrategyV3.create(
        field=field,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetProfileUpdateStrategyV3)
async def public_get_profile_update_strategy_v3_async(
    field: Optional[Union[str, PublicGetProfileUpdateStrategyV3FieldEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Public get profile update strategy by namespace and field. (PublicGetProfileUpdateStrategyV3)

    This API is for public user to get profile update strategy by namespace and field.
    Note: If the config is not found, this API will return a config with unlimited.

    Properties:
        url: /iam/v3/public/namespaces/{namespace}/profileUpdateStrategies

        method: GET

        tags: ["Profile Update Strategy"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        field: (field) OPTIONAL Union[str, FieldEnum] in query

    Responses:
        200: OK - ModelGetProfileUpdateStrategyConfigResponse (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicGetProfileUpdateStrategyV3.create(
        field=field,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )
